@@ -18,6 +18,7 @@
 */
 
 #include "kcompressiondevice.h"
+#include "kcompressiondevice_p.h"
 #include "loggingcategory.h"
 #include <config-compression.h>
 #include "kfilterbase.h"
@@ -36,8 +37,6 @@
 #include "kxzfilter.h"
 #endif
 
-#define BUFFER_SIZE 8*1024
-
 #include <QDebug>
 
 class KCompressionDevicePrivate
@@ -47,7 +46,6 @@ public:
         : bNeedHeader(true)
         , bSkipHeaders(false)
         , bOpenedUnderlyingDevice(false)
-        , bIgnoreData(false)
         , type(KCompressionDevice::None)
         , errorCode(QFileDevice::NoError)
         , deviceReadPos(0)
@@ -60,7 +58,6 @@ public:
     bool bNeedHeader;
     bool bSkipHeaders;
     bool bOpenedUnderlyingDevice;
-    bool bIgnoreData;
     QByteArray buffer; // Used as 'input buffer' when reading, as 'output buffer' when writing
     QByteArray origFileName;
     KFilterBase::Result result;
@@ -92,13 +89,13 @@ KFilterBase *KCompressionDevice::filterForCompressionType(KCompressionDevice::Co
 #if HAVE_BZIP2_SUPPORT
         return new KBzip2Filter;
 #else
-        return 0;
+        return nullptr;
 #endif
     case KCompressionDevice::Xz:
 #if HAVE_XZ_SUPPORT
         return new KXzFilter;
 #else
-        return 0;
+        return nullptr;
 #endif
     case KCompressionDevice::None:
         return new KNoneFilter;
@@ -150,6 +147,9 @@ bool KCompressionDevice::open(QIODevice::OpenMode mode)
         //qCWarning(KArchiveLog) << "KCompressionDevice::open: device is already open";
         return true; // QFile returns false, but well, the device -is- open...
     }
+    if (!d->filter) {
+        return false;
+    }
     d->bOpenedUnderlyingDevice = false;
     //qCDebug(KArchiveLog) << mode;
     if (mode == QIODevice::ReadOnly) {
@@ -246,11 +246,16 @@ bool KCompressionDevice::seek(qint64 pos)
     }
 
     //qCDebug(KArchiveLog) << "reading " << bytesToRead << " dummy bytes";
-    QByteArray dummy(qMin(bytesToRead, qint64(3 * BUFFER_SIZE)), 0);
-    d->bIgnoreData = true;
-    const bool result = (read(dummy.data(), bytesToRead) == bytesToRead);
-    d->bIgnoreData = false;
-    return result;
+    QByteArray dummy(qMin(bytesToRead, qint64(SEEK_BUFFER_SIZE)), 0);
+    while (bytesToRead > 0) {
+        const qint64 bytesToReadThisTime = qMin(bytesToRead, qint64(dummy.size()));
+        const bool result = (read(dummy.data(), bytesToReadThisTime) == bytesToReadThisTime);
+        if (!result) {
+            return false;
+        }
+        bytesToRead -= bytesToReadThisTime;
+    }
+    return true;
 }
 
 bool KCompressionDevice::atEnd() const
@@ -278,15 +283,8 @@ qint64 KCompressionDevice::readData(char *data, qint64 maxlen)
         return -1;
     }
 
-    qint64 outBufferSize;
-    if (d->bIgnoreData) {
-        outBufferSize = qMin(maxlen, static_cast<qint64>(3 * BUFFER_SIZE));
-    } else {
-        outBufferSize = maxlen;
-    }
-    outBufferSize -= dataReceived;
-    qint64 availOut = outBufferSize;
-    filter->setOutBuffer(data, outBufferSize);
+    qint64 availOut = maxlen;
+    filter->setOutBuffer(data, maxlen);
 
     while (dataReceived < maxlen) {
         if (filter->inBufferEmpty()) {
@@ -324,12 +322,8 @@ qint64 KCompressionDevice::readData(char *data, qint64 maxlen)
         }
 
         dataReceived += outReceived;
-        if (!d->bIgnoreData) {  // Move on in the output buffer
-            data += outReceived;
-            availOut = maxlen - dataReceived;
-        } else if (maxlen - dataReceived < outBufferSize) {
-            availOut = maxlen - dataReceived;
-        }
+        data += outReceived;
+        availOut = maxlen - dataReceived;
         if (d->result == KFilterBase::End) {
             // We're actually at the end, no more data to check
             if (filter->device()->atEnd()) {
